@@ -28,6 +28,10 @@
 #include "gnunet-fs-gtk_event-handler.h"
 #include "gnunet-fs-gtk_open-uri.h"
 
+#if HAVE_LIBUNIQUE
+#include <unique/unique.h>
+#endif
+
 /**
  * How many block requests can we have outstanding in parallel at a time by default?
  */
@@ -54,6 +58,22 @@ static struct GNUNET_GTK_MainLoop *ml;
  */
 static struct GNUNET_FS_Handle *fs;
 
+/**
+ * Context for main window.
+ */
+static struct GNUNET_GTK_MainWindowContext main_context;
+
+
+#if HAVE_LIBUNIQUE
+static UniqueApp *unique_app;
+#endif
+
+struct GNUNET_GTK_MainWindowContext *
+GNUNET_FS_GTK_get_main_context ()
+{
+  return &main_context;
+}
+
 
 /**
  * Return handle for file-sharing operations.
@@ -162,31 +182,15 @@ main_window_save_position (GtkWidget *main_window)
  *
  * @param main_window main window widget
  */
-void
-GNUNET_FS_GTK_main_window_drag_end (GtkWidget *main_window,
-				    GdkDragContext *drag_context,
-				    gpointer user_data)
-{
-  struct GNUNET_GTK_MainWindowContext *main_context = user_data;
-
-  main_window_save_position (main_context->main_window);
-}
-
-
-/**
- * Obtains main window position and size before it's destroyed
- * and saves these into user's config file.
- *
- * @param main_window main window widget
- */
-void
-GNUNET_FS_GTK_main_window_size_request (GtkWidget *main_window,
-					GtkRequisition *requisition,
-					gpointer user_data)
+gboolean
+GNUNET_GTK_main_window_configure_event_cb (GtkWidget *main_window,
+				           GdkEventConfigure *event,
+				           gpointer user_data)
 {
   struct GNUNET_GTK_MainWindowContext *main_context = user_data;
 
   main_window_save_position (main_context->main_window);
+  return FALSE;
 }
 
 
@@ -199,13 +203,17 @@ GNUNET_FS_GTK_main_window_size_request (GtkWidget *main_window,
  */
 static void
 shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
-{
-  GNUNET_GTK_main_loop_quit (ml);
+{  
+  GNUNET_GTK_tray_icon_destroy ();
   if (fs != NULL)
   {
     GNUNET_FS_stop (fs);
     fs = NULL;
   }
+  GNUNET_FS_GTK_close_uri_tab_ ();
+  if (NULL != ml)
+    GNUNET_GTK_main_loop_quit (ml);
+  ml = NULL;
 }
 
 
@@ -217,14 +225,9 @@ shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
  */
 void
 GNUNET_FS_GTK_menu_quit_activate_cb (GtkMenuItem *object,
-				  gpointer user_data)
+				     gpointer user_data)
 {
-  struct GNUNET_GTK_MainWindowContext *main_context = user_data;
-
-  GNUNET_GTK_tray_icon_destroy ();
-  GNUNET_SCHEDULER_add_with_priority (GNUNET_SCHEDULER_PRIORITY_IDLE,
-                                      &shutdown_task, NULL);
-  GNUNET_free (main_context);
+  GNUNET_SCHEDULER_shutdown ();
 }
 
 
@@ -236,15 +239,21 @@ GNUNET_FS_GTK_menu_quit_activate_cb (GtkMenuItem *object,
  */
 void
 GNUNET_FS_GTK_delete_event_cb (GtkWidget *object,
-			    GdkEvent *event,
-			    gpointer user_data)
+			       GdkEvent *event,
+			       gpointer user_data)
 {
-  struct GNUNET_GTK_MainWindowContext *main_context = user_data;
+  /* GNUNET_FS_GTK_delete_event_cb will eventually be called if we shut down
+   * the scheduler, because shutting it down will make GTK delete the main
+   * window. On the other hand, deleting the main window first (clicking on X
+   * button) will not trigger scheduler shutdown, unlike
+   * GNUNET_FS_GTK_menu_quit_activate_cb(). So we shut it down here again,
+   * just to be sure it is dead (if it isn't, application will hang up).
+   */
+  GNUNET_SCHEDULER_shutdown ();
 
   GNUNET_GTK_tray_icon_destroy ();
   GNUNET_SCHEDULER_add_with_priority (GNUNET_SCHEDULER_PRIORITY_IDLE,
                                       &shutdown_task, NULL);
-  GNUNET_free (main_context);
 }
 
 
@@ -306,6 +315,63 @@ GNUNET_FS_GTK_main_window_button_press_event (GtkWidget * widget,
 }
 
 
+#if HAVE_LIBUNIQUE
+/**
+ * Function called whenever a second gnunet-fs-gtk process is started
+ * with additional arguments for us.
+ *
+ * @param app unique handle
+ * @param command command that was given
+ * @param message_data command line message data
+ * @param time_ timestamp of the event
+ * @param user_data our 'main context'
+ * @return response code for original process
+ */
+static UniqueResponse
+unique_app_message_cb (UniqueApp *app,
+		       gint command,
+		       UniqueMessageData *message_data,
+		       guint time_,
+		       gpointer user_data)
+{
+  struct GNUNET_GTK_MainWindowContext *main_context = user_data;
+
+  /* raise the window */
+  gtk_window_present_with_time (GTK_WINDOW (main_context->main_window), time_);
+
+  switch (command)
+    {
+    case UNIQUE_NEW:
+      /* this is unexpected... */
+      GNUNET_break (0);
+      break;
+    case UNIQUE_OPEN:
+      {
+        gchar **uris;
+        gint n_uris;
+	gint i;
+
+        uris = unique_message_data_get_uris (message_data);
+        n_uris = g_strv_length (uris);
+        for (i = 0; i < n_uris; i++)
+	{
+	  if (GNUNET_OK !=
+	      GNUNET_FS_GTK_handle_uri_string (uris[i],
+					       1 /* anonymity level */))
+	    return UNIQUE_RESPONSE_PASSTHROUGH;
+	}
+        g_strfreev (uris);
+      }
+      break;
+    case UNIQUE_ACTIVATE:
+      break;
+    default:
+      break;
+    }
+  return UNIQUE_RESPONSE_OK;
+}
+#endif
+
 
 /**
  * Actual main function run right after GNUnet's scheduler
@@ -317,37 +383,45 @@ GNUNET_FS_GTK_main_window_button_press_event (GtkWidget * widget,
 static void
 run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
-  unsigned long long dl_parallel, req_parallel, window_x, window_y,
-      window_width, window_height;
+  unsigned long long dl_parallel;
+  unsigned long long req_parallel;
+  unsigned long long window_x;
+  unsigned long long window_y;
+  unsigned long long window_width;
+  unsigned long long window_height;
   int maximized;
-  struct GNUNET_GTK_MainWindowContext *main_context;
 
-  main_context = GNUNET_malloc (sizeof (struct GNUNET_GTK_MainWindowContext));
   ml = cls;
   /* setup main context */
-  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (cls, main_context))
-  {
-    GNUNET_free (main_context);
+  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (cls, &main_context))
     return;
-  }
-  main_context->builder = GNUNET_GTK_main_loop_get_builder (cls);
-  main_context->cfg = GNUNET_GTK_main_loop_get_configuration (cls);
-  main_context->search_ns_treestore = GTK_TREE_STORE (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_treestore"));
-  main_context->main_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window"));
-  main_context->ns_selector_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_treeview"));
-  main_context->ns_selector_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_window"));
-  main_context->ns_dropdown_button = GTK_TOGGLE_BUTTON (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_dropdown_button"));
-  main_context->search_ns_label = GTK_LABEL (GNUNET_FS_GTK_get_main_window_object ("main_window_search_selected_namespace_label"));
+  main_context.builder = GNUNET_GTK_main_loop_get_builder (cls);
+  main_context.cfg = GNUNET_GTK_main_loop_get_configuration (cls);
+  main_context.search_ns_treestore = GTK_TREE_STORE (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_treestore"));
+  main_context.main_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window"));
+  main_context.ns_selector_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_treeview"));
+  main_context.ns_selector_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_window"));
+  main_context.ns_dropdown_button = GTK_TOGGLE_BUTTON (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_dropdown_button"));
+  main_context.search_ns_label = GTK_LABEL (GNUNET_FS_GTK_get_main_window_object ("main_window_search_selected_namespace_label"));
 
-  main_context->search_entry = GTK_ENTRY (GNUNET_FS_GTK_get_main_window_object ("main_window_search_entry"));
+  main_context.search_entry = GTK_ENTRY (GNUNET_FS_GTK_get_main_window_object ("main_window_search_entry"));
 
-  main_context->anonymity_combo = GTK_COMBO_BOX (GNUNET_FS_GTK_get_main_window_object ("main_window_search_anonymity_combobox"));
-  main_context->anonymity_level_liststore = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("anonymity_level_liststore"));
+  main_context.anonymity_combo = GTK_COMBO_BOX (GNUNET_FS_GTK_get_main_window_object ("main_window_search_anonymity_combobox"));
+  main_context.anonymity_level_liststore = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("anonymity_level_liststore"));
 
-  main_context->preview_image = GTK_IMAGE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_preview_image"));
-  main_context->md_liststore = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_meta_data_list_store"));
-  main_context->md_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_metadata_treeview"));
-  main_context->ns_callback_registered = GNUNET_NO;
+  main_context.preview_image = GTK_IMAGE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_preview_image"));
+  main_context.md_liststore = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_meta_data_list_store"));
+  main_context.md_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_metadata_treeview"));
+  main_context.ns_callback_registered = GNUNET_NO;
+
+  main_context.download_location_chooser = GTK_FILE_CHOOSER (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_location_chooser"));
+  main_context.download_name_entry = GTK_ENTRY (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_filename_entry"));
+  main_context.download_anonymity_combo = GTK_COMBO_BOX (GNUNET_FS_GTK_get_main_window_object ("main_window_download_anonymity_combobox"));
+  main_context.download_recursive_checkbutton = GTK_CHECK_BUTTON (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_recursive_checkbox"));
+  main_context.download_download_button = GTK_BUTTON (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_download_button"));
+  main_context.download_panel = GTK_VBOX (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_vbox"));
+
+  main_context.notebook = GTK_NOTEBOOK (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_notebook"));
 
   GNUNET_GTK_set_icon_search_path ();
   GNUNET_GTK_setup_nls ();
@@ -360,50 +434,51 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   g_object_set (gtk_settings_get_default (), "gtk-button-images", TRUE, NULL);
 
   /* setup main window */
-  maximized = GNUNET_CONFIGURATION_get_value_yesno (main_context->cfg,
+  maximized = GNUNET_CONFIGURATION_get_value_yesno (main_context.cfg,
 						    "gnunet-gtk", "MAIN_WINDOW_MAXIMIZED");
   if (GNUNET_SYSERR == maximized)
     maximized = GNUNET_YES;
   if ( (GNUNET_NO == maximized) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
 							    "MAIN_WINDOW_X", &window_x)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
 							    "MAIN_WINDOW_Y", &window_y)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
 							    "MAIN_WINDOW_WIDTH", &window_width)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
 							    "MAIN_WINDOW_HEIGHT", &window_height)) )
   {
-    gtk_window_move (GTK_WINDOW (main_context->main_window), window_x, window_y);
-    gtk_window_resize (GTK_WINDOW (main_context->main_window), window_width, window_height);
+    gtk_window_move (GTK_WINDOW (main_context.main_window), window_x, window_y);
+    gtk_window_resize (GTK_WINDOW (main_context.main_window), window_width, window_height);
   }
   else
   {
     /* If anything is wrong - play safe and show it maximized */
-    gtk_window_maximize (GTK_WINDOW (main_context->main_window));
+    gtk_window_maximize (GTK_WINDOW (main_context.main_window));
   }
   
   /* Allow multiple selection in metadata view; */
   /* FIXME-GTK3: this can be done within (modern versions of) glade */
-  gtk_tree_selection_set_mode (gtk_tree_view_get_selection (main_context->md_treeview),
+  gtk_tree_selection_set_mode (gtk_tree_view_get_selection (main_context.md_treeview),
                                GTK_SELECTION_MULTIPLE);
 
-  GNUNET_GTK_tray_icon_create (GTK_WINDOW (main_context->main_window),
+  GNUNET_GTK_tray_icon_create (ml,
+			       GTK_WINDOW (main_context.main_window),
                                "gnunet-fs-gtk",
                                "gnunet-fs-gtk");
 
   /* FIXME: should these '1's be here? Maybe better to put them into
    * default config files?
    */
-  if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-fs-gtk",
+  if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
       "MAX_PARALLEL_DOWNLOADS", &dl_parallel))
     dl_parallel = DEFAULT_MAX_PARALLEL_DOWNLOADS;
-  if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_number (main_context->cfg, "gnunet-fs-gtk",
+  if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
       "MAX_PARALLEL_REQUESTS", &req_parallel))
     req_parallel = DEFAULT_MAX_PARALLEL_REQUESTS;
 
   /* initialize file-sharing */
-  fs = GNUNET_FS_start (main_context->cfg, "gnunet-fs-gtk",
+  fs = GNUNET_FS_start (main_context.cfg, "gnunet-fs-gtk",
 			&GNUNET_GTK_fs_event_handler, NULL, 
 			GNUNET_FS_FLAGS_PERSISTENCE | GNUNET_FS_FLAGS_DO_PROBES,
                         GNUNET_FS_OPTIONS_DOWNLOAD_PARALLELISM,
@@ -414,37 +489,79 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   if (NULL == fs)
   {
     GNUNET_GTK_main_loop_quit (cls);
-    GNUNET_free (main_context);
     return;
   }
 
-  GNUNET_GTK_main_window_refresh_ns_list (main_context);
-
+  GNUNET_GTK_main_window_refresh_ns_list (&main_context);
+#if HAVE_LIBUNIQUE
+  unique_app_watch_window (unique_app, GTK_WINDOW (main_context.main_window));
+  g_signal_connect (unique_app, "message-received",
+		    G_CALLBACK (unique_app_message_cb), &main_context);
+#endif
   /* make GUI visible */
   if (!tray_only)
   {
-    gtk_widget_show (main_context->main_window);
-    gtk_window_present (GTK_WINDOW (main_context->main_window));
+    gtk_widget_show (main_context.main_window);
+    gtk_window_present (GTK_WINDOW (main_context.main_window));
   }
-
+  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
+				&shutdown_task, NULL);
 }
 
 
 int
-main (int argc, char *const *argv)
+main (int argc, char **argv)
 {
   static struct GNUNET_GETOPT_CommandLineOption options[] = {
     {'t', "tray", NULL,
      gettext_noop ("start in tray mode"), 0,
      &GNUNET_GETOPT_set_one, &tray_only},
     GNUNET_GETOPT_OPTION_END
-  };
+  };  
+#if HAVE_LIBUNIQUE
+  int arge;
 
+  gtk_init (&argc, &argv);
+  unique_app = unique_app_new ("org.gnunet.gnunet-fs-gtk", NULL);
+  if (unique_app_is_running (unique_app))
+  {
+    UniqueResponse response;
+
+    arge = GNUNET_GETOPT_run ("gnunet-fs-gtk",
+			      options,
+			      argc, argv);
+    response = unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
+    while (arge < argc)
+    {
+      UniqueMessageData *msg;
+
+      msg = unique_message_data_new ();
+      unique_message_data_set_text (msg, argv[arge], strlen (argv[arge])+1);      
+      if (UNIQUE_RESPONSE_OK == response)
+	response = unique_app_send_message (unique_app,
+					    UNIQUE_OPEN,
+					    msg);
+      unique_message_data_free (msg);
+      arge++;
+    }
+    g_object_unref (unique_app);
+
+    return (UNIQUE_RESPONSE_OK == response) ? 0 : 1;
+  }
+#endif
   if (GNUNET_OK !=
       GNUNET_GTK_main_loop_start ("gnunet-fs-gtk", "GTK GUI for GNUnet", argc,
                                   argv, options,
                                   "gnunet_fs_gtk_main_window.glade", &run))
+  {
+#if HAVE_LIBUNIQUE
+    g_object_unref (unique_app);
+#endif
     return 1;
+  }
+#if HAVE_LIBUNIQUE
+  g_object_unref (unique_app);
+#endif
   return 0;
 }
 
