@@ -58,6 +58,15 @@ struct GNUNET_GTK_MainWindowContext
   int ns_callback_registered;
   GtkWindow *ns_manager;
   gulong ns_manager_delete_handler_id;
+
+  GtkFileChooser *download_location_chooser;
+  GtkEntry *download_name_entry;
+  GtkComboBox *download_anonymity_combo;
+  GtkCheckButton *download_recursive_checkbutton;
+  GtkButton *download_download_button;
+  GtkVBox *download_panel;
+
+  GtkNotebook *notebook;
 };
 
 
@@ -168,6 +177,12 @@ enum GNUNET_GTK_FS_MAIN_WINDOW_SearchMimeModelColumns
     GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_MIME_MC_FREEDOM = 4,
   };
 
+/**
+ * Get main context.
+ */
+struct GNUNET_GTK_MainWindowContext *
+GNUNET_FS_GTK_get_main_context ();
+
 
 /**
  * Get our configuration.
