@@ -265,17 +265,24 @@ GNUNET_GTK_select_pseudonym_dialog_delete_event_cb (GtkWidget *widget,
 static void
 add_to_list (void *cls,
 	     const char *name,
-	     const GNUNET_HashCode * id)
+	     const struct GNUNET_HashCode * id)
 {
   GtkListStore *ls = cls;
   GtkTreeIter iter;
+  struct GNUNET_FS_Namespace *ns;
 
+  ns = GNUNET_FS_namespace_create (GNUNET_FS_GTK_get_fs_handle (),
+				   name);
+  if (NULL == ns)
+  {
+    GNUNET_break (0);
+    return;
+  }
   gtk_list_store_insert_with_values (ls, &iter, -1,
                                      FS_SELECT_PSEUDONYM_NAMESPACE_MC_NAME,
                                      name,
                                      FS_SELECT_PSEUDONYM_NAMESPACE_MC_HANDLE,
-                                     GNUNET_FS_namespace_create (GNUNET_FS_GTK_get_fs_handle (),
-                                                                 name),
+                                     ns,
                                      -1);
 }
 
