@@ -322,8 +322,8 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri,
   GtkLabel *sel_namespace_label;
   GtkTreePath *treepath;
   GtkEntry *query_entry;
-  GNUNET_HashCode *nsid;
-  GNUNET_HashCode want;
+  struct GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode want;
   
   if (GNUNET_FS_uri_test_chk (uri) || GNUNET_FS_uri_test_loc (uri))
   {
@@ -366,7 +366,7 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri,
     while ( ( (NULL == nsid) ||
 	      (0 != memcmp (nsid,
 			    &want,
-			    sizeof (GNUNET_HashCode))) ) &&
+			    sizeof (struct GNUNET_HashCode))) ) &&
 	    (gtk_tree_model_iter_next (namespace_treestore, &iter)) )
       gtk_tree_model_get (namespace_treestore, &iter,
                           GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_NAMESPACE_MC_KEY,
@@ -375,7 +375,7 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri,
     if ( (NULL == nsid) ||
 	 (0 != memcmp (nsid,
 		       &want,
-		       sizeof (GNUNET_HashCode))) )
+		       sizeof (struct GNUNET_HashCode))) )
     {
       /* namespace unknown / not in list!? */
       GNUNET_break (0);
@@ -448,7 +448,7 @@ static int largest_namespace_rating = INT_MIN;
  */
 static int
 find_largest_namespace_rating_iterator (void *cls,
-    const GNUNET_HashCode *pseudonym, const char *name,
+    const struct GNUNET_HashCode *pseudonym, const char *name,
     const char *unique_name,
     const struct GNUNET_CONTAINER_MetaData *md, int rating)
 {
@@ -545,5 +545,34 @@ GNUNET_GTK_get_reference_from_iter (GtkTreeModel *model, GtkTreeIter *iter)
   return result;
 }
 
+gboolean
+GNUNET_GTK_tree_model_get_next_flat_iter (GtkTreeModel *model, GtkTreeIter *iter, gboolean allow_children, GtkTreeIter *next_iter)
+{
+  GtkTreeIter current_iter = *iter;
+  while (TRUE)
+  {
+    GtkTreeIter tmp_iter;
+
+    tmp_iter = current_iter;
+    if (gtk_tree_model_iter_next (model, &tmp_iter))
+    {
+      *next_iter = tmp_iter;
+      return TRUE;
+    }
+    if (allow_children)
+    {
+      if (gtk_tree_model_iter_children (model, &tmp_iter, &current_iter))
+      {
+        *next_iter = tmp_iter;
+        return TRUE;
+      }
+    }
+    allow_children = FALSE;
+    if (!gtk_tree_model_iter_parent (model, &tmp_iter, &current_iter))
+      return FALSE;
+    current_iter = tmp_iter;
+  }
+  return FALSE;
+}
 
 /* end of gnunet-fs-gtk-common.c */
