@@ -194,5 +194,26 @@ GNUNET_GTK_get_iter_from_reference (GtkTreeRowReference *rr, GtkTreeIter *iter);
 GtkTreeRowReference *
 GNUNET_GTK_get_reference_from_iter (GtkTreeModel *model, GtkTreeIter *iter);
 
+/**
+ * Fills "next_iter" with iterator for an item that comes next in the tree
+ * after "iter".
+ * Next item is, in order of precedence:
+ * 1) First child of "iter", if "iter" has children and "allow_children"
+ *   is enabled.
+ * 2) Next sibling of "iter", unless "iter" is the last sibling.
+ * If none of those are present, function recursively checks parents of
+ *   "iter" until it finds next item or runs out of parents.
+ *
+ * @param model a model to reference
+ * @param iter an iter that points to current row in the model
+ * @param allow_children whether child of "iter" is considered to be next.
+ * @param next_iter will be filled with the next row in the model on success
+ * @return TRUE if next_iter is set to a valid iter,
+ *         FALSE if ran out of parents
+ */
+gboolean
+GNUNET_GTK_tree_model_get_next_flat_iter (GtkTreeModel *model,
+    GtkTreeIter *iter, gboolean allow_children, GtkTreeIter *next_iter);
+
 #endif
 /* end of gnunet-fs-gtk-common.h */
