@@ -107,6 +107,11 @@ struct SearchTab
    */
   GtkTreeStore *ts;
 
+  /**
+   * Animation handle associated with the tree store.
+   */
+  struct GNUNET_FS_AnimationTreeViewHandle *atv;
+
   /**
    * Number of results we got for this search.
    */
@@ -171,7 +176,6 @@ struct DownloadEntry
    * Has the download completed (or failed)?
    */
   int is_done;
-
 };
 
 
@@ -244,6 +248,9 @@ GNUNET_GTK_add_search_result (struct SearchTab *tab,
                               uint32_t applicability_rank);
 
 
+void
+GNUNET_FS_GTK_set_item_downloaded_name (GtkTreeStore *ts, GtkTreeRowReference *rr, gchar *filename);
+
 /**
  * Notification of FS to a client about the progress of an
  * operation.  Callbacks of this type will be used for uploads,
@@ -264,5 +271,12 @@ GNUNET_GTK_fs_event_handler (void *cls,
                              const struct GNUNET_FS_ProgressInfo *info);
 
 
+/**
+ * Close the 'uri_tab'.
+ */ 
+void
+GNUNET_FS_GTK_close_uri_tab_ ();
+
+
 #endif
 /* end of gnunet-fs-gtk-event_handler.h */
