@@ -292,14 +292,14 @@ GNUNET_FS_GTK_search_namespace_dropdown_button_toggled_cb (GtkToggleButton *
  * @return GNUNET_OK to continue iteration, GNUNET_SYSERR to abort
  */
 static int
-add_namespace_to_ts (void *cls, const GNUNET_HashCode * pseudonym,
+add_namespace_to_ts (void *cls, const struct GNUNET_HashCode * pseudonym,
                      const char *name, const char *unique_name,
                      const struct GNUNET_CONTAINER_MetaData *md, int rating)
 {
   GtkTreeStore *ts = cls;
   char *root;
   char *ns_name, *unique_ns_name;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   char *description;
   int desc_is_a_dup;
   char *uris;
@@ -315,7 +315,7 @@ add_namespace_to_ts (void *cls, const GNUNET_HashCode * pseudonym,
   unique_ns_name = GNUNET_PSEUDONYM_name_uniquify (
       GNUNET_FS_GTK_get_configuration (), pseudonym, ns_name, NULL);
   GNUNET_free (ns_name);
-  nsid = GNUNET_malloc (sizeof (GNUNET_HashCode));
+  nsid = GNUNET_malloc (sizeof (struct GNUNET_HashCode));
   *nsid = *pseudonym;
   root = NULL;
   uris = GNUNET_CONTAINER_meta_data_get_by_type (md, EXTRACTOR_METATYPE_URI);
@@ -350,7 +350,7 @@ GNUNET_GTK_main_window_refresh_ns_list (struct GNUNET_GTK_MainWindowContext *mai
 {
   GtkTreeIter iter;
   GtkTreePath *treepath;
-  GNUNET_HashCode *key = NULL, *selected_ns_id;
+  struct GNUNET_HashCode *key = NULL, *selected_ns_id;
 
   gboolean found = FALSE;
   gchar *value = NULL;
@@ -377,8 +377,8 @@ GNUNET_GTK_main_window_refresh_ns_list (struct GNUNET_GTK_MainWindowContext *mai
   selected_ns_id = NULL;
   if (key != NULL)
   {
-    selected_ns_id = GNUNET_malloc (sizeof (GNUNET_HashCode));
-    memcpy (selected_ns_id, key, sizeof (GNUNET_HashCode));
+    selected_ns_id = GNUNET_malloc (sizeof (struct GNUNET_HashCode));
+    memcpy (selected_ns_id, key, sizeof (struct GNUNET_HashCode));
   }
 
   if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (
@@ -429,7 +429,7 @@ GNUNET_GTK_main_window_refresh_ns_list (struct GNUNET_GTK_MainWindowContext *mai
                           -1);
       if (selected_ns_id == NULL)
         found = TRUE;
-      else if (key != NULL && memcmp (key, selected_ns_id, sizeof (GNUNET_HashCode)) == 0)
+      else if (key != NULL && memcmp (key, selected_ns_id, sizeof (struct GNUNET_HashCode)) == 0)
         found = TRUE;
       if (found || (TRUE != gtk_tree_model_iter_next (GTK_TREE_MODEL (
         main_ctx->search_ns_treestore), &iter)))
