@@ -31,7 +31,7 @@
 /**
  * Start a search.
  *
- * @param builder the main dialog builder
+ * @param builder the main window context
  */
 static void
 start_search (struct GNUNET_GTK_MainWindowContext *main_ctx)
@@ -39,7 +39,7 @@ start_search (struct GNUNET_GTK_MainWindowContext *main_ctx)
   guint anonymity_level;
   gchar *keywords;
   gchar *mime_keyword;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   struct GNUNET_FS_Uri *uri;
 
   /* get anonymity level */
@@ -149,7 +149,7 @@ start_search (struct GNUNET_GTK_MainWindowContext *main_ctx)
  * User clicked on the 'search' button in the main window.
  *
  * @param button the search button
- * @param user_data the main dialog builder
+ * @param user_data the main window context builder
  */
 void
 main_window_search_button_clicked_cb (GtkButton * button, 
@@ -159,14 +159,13 @@ main_window_search_button_clicked_cb (GtkButton * button,
   start_search (main_ctx);
 }
 
-
 /**
  * User pushed a key (possibly ENTER) in the search entry widget.
  * Start the search if it was ENTER.
  *
  * @param widget the entry widget
  * @param event the key stroke
- * @param user_data the builder of the main window
+ * @param user_data the main window context
  * @return FALSE if this was not ENTER, TRUE if it was
  */
 gboolean
