@@ -199,7 +199,7 @@ GNUNET_GTK_namespace_manager_known_namespaces_delete_button_clicked_cb (
 {
   struct GNUNET_GTK_NamespaceManagerContext *ctx = user_data;
   GtkTreeIter iter;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   GtkTreeRowReference *order_row, *known_row;
   struct GNUNET_CONTAINER_MetaData *md;
 
@@ -238,7 +238,7 @@ GNUNET_GTK_namespace_manager_known_namespaces_swap_rank_button_clicked_cb (
 {
   struct GNUNET_GTK_NamespaceManagerContext *ctx = user_data;
   GtkTreeIter known_iter, order_iter;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   int32_t old_rank, new_rank;
   GtkTreeRowReference *order_row, *known_row;
 
@@ -495,7 +495,7 @@ GNUNET_GTK_namespace_manager_name_text_edited_cb (
     if (gtk_tree_model_get_iter (GTK_TREE_MODEL (ctx->known_ns_store),
         &iter, tree_path))
     {
-      GNUNET_HashCode *nsid;
+      struct GNUNET_HashCode *nsid;
       gtk_tree_model_get (GTK_TREE_MODEL (ctx->known_ns_store), &iter,
           GNUNET_GTK_KNOWN_NAMESPACES_IDENTIFIER_BIN_COLUMN, &nsid,
           GNUNET_GTK_KNOWN_NAMESPACES_ORDER_ROW_REFERENCE_COLUMN, &order_row,
@@ -590,18 +590,18 @@ GNUNET_GTK_namespace_manager_namespace_details_value_text_edited_cb (
  * @return GNUNET_OK to continue iteration, GNUNET_SYSERR to abort
  */
 static int
-populate_known_ns_list (void *cls, const GNUNET_HashCode *pseudonym,
+populate_known_ns_list (void *cls, const struct GNUNET_HashCode *pseudonym,
     const char *name, const char *unique_name,
     const struct GNUNET_CONTAINER_MetaData *md, int rating)
 {
   struct GNUNET_GTK_NamespaceManagerContext *ctx = cls;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   struct GNUNET_CRYPTO_HashAsciiEncoded identifier;
   GtkTreeIter known_iter, order_iter;
   struct GNUNET_CONTAINER_MetaData *md_copy;
   char *non_null_name, *non_null_unique_name;
 
-  nsid = GNUNET_malloc (sizeof (GNUNET_HashCode));
+  nsid = GNUNET_malloc (sizeof (struct GNUNET_HashCode));
   *nsid = *pseudonym;
 
   GNUNET_CRYPTO_hash_to_enc (nsid, &identifier);
@@ -652,7 +652,7 @@ apply_known_ns_changes (struct GNUNET_GTK_NamespaceManagerContext *ctx)
   GtkTreeIter iter;
   gint i;
   gint row_count;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   char *name;
   int32_t rank;
   struct GNUNET_CONTAINER_MetaData *md;
@@ -686,7 +686,7 @@ free_lists_contents (struct GNUNET_GTK_NamespaceManagerContext *ctx)
   GtkTreeIter iter;
   gint i;
   gint row_count;
-  GNUNET_HashCode *nsid;
+  struct GNUNET_HashCode *nsid;
   GtkTreeRowReference *order_row;
   struct GNUNET_CONTAINER_MetaData *md;
 
@@ -764,13 +764,13 @@ static gboolean
 mark_as_mine (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
     gpointer data)
 {
-  const GNUNET_HashCode *mine_id = data;
-  const GNUNET_HashCode *ns_id;
+  const struct GNUNET_HashCode *mine_id = data;
+  const struct GNUNET_HashCode *ns_id;
 
   gtk_tree_model_get (model, iter,
       GNUNET_GTK_KNOWN_NAMESPACES_IDENTIFIER_BIN_COLUMN, &ns_id, -1);
 
-  if (memcmp (ns_id, mine_id, sizeof (GNUNET_HashCode)) != 0)
+  if (memcmp (ns_id, mine_id, sizeof (struct GNUNET_HashCode)) != 0)
     return FALSE;
 
   gtk_list_store_set (GTK_LIST_STORE (model), iter,
@@ -789,7 +789,7 @@ mark_as_mine (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
  */
 static void
 mark_namespaces_as_mine (void *cls, const char *name,
-    const GNUNET_HashCode * id)
+    const struct GNUNET_HashCode * id)
 {
   struct GNUNET_GTK_NamespaceManagerContext *ctx = cls;
 
