@@ -34,7 +34,7 @@
  * Be very verbose when reporting progress (usually bad as it takes more time
  * to display this than to make progress).
  */
-#define VERBOSE_PROGRESS GNUNET_NO
+#define VERBOSE_PROGRESS GNUNET_EXTRA_LOGGING
 
 
 /**
@@ -798,7 +798,7 @@ GNUNET_GTK_master_publish_dialog_up_button_clicked_cb (GtkWidget * dummy,
     GNUNET_break (0);
     return;
   }
-  if (! gtk_tree_model_iter_parent (ctx->file_info_treemodel, &parent, &iter))
+  if (gtk_tree_model_iter_parent (ctx->file_info_treemodel, &parent, &iter))
   {
     GNUNET_assert (TRUE == gtk_tree_model_iter_children (ctx->file_info_treemodel, &pred, &parent));
     pprev = &parent;
@@ -807,8 +807,8 @@ GNUNET_GTK_master_publish_dialog_up_button_clicked_cb (GtkWidget * dummy,
   {
     GNUNET_break (0);
     return;
-  } else
-    pprev = NULL;    
+  } 
+  pprev = NULL;    
   /* iterate over 'next' of pred to find out who our predecessor is! */
   while (GNUNET_YES != gtk_tree_iter_equals (ctx->file_info_treemodel, &pred, &iter))
   {
@@ -1120,7 +1120,7 @@ add_item (struct AddDirClientContext *adcc,
   struct stat sbuf;
   uint64_t fsize;
   
-  if (0 != stat (item->filename,
+  if (0 != STAT (item->filename,
 		 &sbuf))
   {
     GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR, "stat", item->filename);
@@ -1663,16 +1663,21 @@ update_treeview_after_edit (void *cls, struct GNUNET_FS_FileInformation *fi,
 			    void **client_info)
 {
   struct EditPublishContext *epc = cls;
-  
+  char *name;
+              
+  name = GNUNET_CONTAINER_meta_data_get_by_type (meta,
+						 EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME);
   gtk_tree_store_set (GTK_TREE_STORE (epc->tm), &epc->iter,
                       PUBLISH_MC_DO_INDEX, *do_index,
-                      PUBLISH_MC_ANONYMITY_LEVEL, (guint) bo->anonymity_level,
+                      PUBLISH_MC_FILENAME, name, 
+		      PUBLISH_MC_ANONYMITY_LEVEL, (guint) bo->anonymity_level,
                       PUBLISH_MC_PRIORITY, (guint) bo->content_priority,
                       PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
                       (guint64) bo->expiration_time.abs_value,
                       PUBLISH_MC_REPLICATION_LEVEL,
                       (guint) bo->replication_level,
                       -1);
+  GNUNET_free (name);
   return GNUNET_SYSERR;
 }
 
@@ -1925,6 +1930,92 @@ GNUNET_GTK_master_publish_dialog_execute_button_clicked_cb (GtkButton * button,
 }
 
 
+/**
+ * Function called on entries in a GNUNET_FS_FileInformation publish-structure.
+ *
+ * @param cls closure, unused
+ * @param fi the entry in the publish-structure
+ * @param length length of the file or directory
+ * @param meta metadata for the file or directory (can be modified)
+ * @param uri pointer to the keywords that will be used for this entry (will be set to NULL)
+ * @param bo block options (can be modified)
+ * @param do_index should we index (can be modified)
+ * @param client_info pointer to client context set upon creation (can be modified)
+ * @return GNUNET_OK (to continue)
+ */
+static int
+clear_keywords_in_file_information (void *cls,
+				    struct GNUNET_FS_FileInformation *fi,
+				    uint64_t length,
+				    struct
+				    GNUNET_CONTAINER_MetaData *
+				    meta,
+				    struct GNUNET_FS_Uri ** uri,
+				    struct GNUNET_FS_BlockOptions
+				    * bo, int *do_index,
+				    void **client_info)
+{
+  if (NULL == *uri)
+    return GNUNET_OK;
+  GNUNET_FS_uri_destroy (*uri);
+  *uri = NULL;
+  return GNUNET_OK;
+}
+
+
+/**
+ * Remove all of the keywords from the file information in the tree store
+ *
+ * @param tm tree model / store
+ * @param iter current position in the recursion
+ */
+static void
+clear_keywords_from_file_information_in_tree_store (GtkTreeModel * tm,
+						    GtkTreeIter * iter)
+{
+  GtkTreeIter child;
+  struct GNUNET_FS_FileInformation *fip;
+  
+  gtk_tree_model_get (tm, iter, PUBLISH_MC_FILE_INFORMATION_STRUCT, &fip, -1);
+  if (NULL != fip)
+  {
+    GNUNET_FS_file_information_inspect (fip, &clear_keywords_in_file_information, NULL);
+  }
+  /* recursively clean up children */
+  if (gtk_tree_model_iter_children (tm, &child, iter))
+  {
+    do
+    {
+      clear_keywords_from_file_information_in_tree_store (tm, &child);
+    }
+    while (gtk_tree_model_iter_next (tm, &child));
+  }
+}
+
+
+/**
+ * The user pushed the 'clear' button.  Remove all keywords.
+ *
+ * @param button the button that was clicked
+ * @param user_data master publishing dialog context of our window
+ */
+void
+GNUNET_GTK_master_publish_dialog_clear_button_clicked_cb (GtkButton * button,
+							  gpointer user_data)
+{
+  struct MainPublishingDialogContext *ctx = user_data;
+  GtkTreeIter iter;
+
+  /* clear keywords from 'tm' */
+  if (gtk_tree_model_get_iter_first (ctx->file_info_treemodel, &iter))
+    do
+    {
+      clear_keywords_from_file_information_in_tree_store (ctx->file_info_treemodel, &iter);
+    }
+    while (gtk_tree_model_iter_next (ctx->file_info_treemodel, &iter));
+}
+
+
 /**
  * The user pushed the 'cancel' button.  Close the master publish dialog.
  *
@@ -2026,7 +2117,7 @@ add_updateable_to_ts (void *cls, const char *last_id,
   GtkTreeIter titer;
   char *desc;
   int desc_is_a_dup;
-  GNUNET_HashCode hc;
+  struct GNUNET_HashCode hc;
 
   uc->update_called = GNUNET_YES;
   GNUNET_CRYPTO_hash (last_id, strlen (last_id), &hc);
@@ -2098,7 +2189,7 @@ add_updateable_to_ts (void *cls, const char *last_id,
  * @param id identity of the namespace to add
  */
 static void
-add_namespace_to_ts (void *cls, const char *name, const GNUNET_HashCode * id)
+add_namespace_to_ts (void *cls, const char *name, const struct GNUNET_HashCode * id)
 {
   GtkTreeStore *ts = cls;
   struct UpdateableContext uc;
@@ -2109,6 +2200,11 @@ add_namespace_to_ts (void *cls, const char *name, const GNUNET_HashCode * id)
   uc.ts = ts;
   uc.ns = GNUNET_FS_namespace_create (GNUNET_FS_GTK_get_fs_handle (), name);
   uc.update_called = GNUNET_NO;
+  if (NULL == uc.ns)
+  {
+    GNUNET_break (0);
+    return;
+  }
   gtk_tree_store_insert_with_values (ts, &iter, NULL, G_MAXINT, 
                                      PSEUDONYM_MC_LOCAL_NAME, name,
                                      PSEUDONYM_MC_NAMESPACE_HANDLE, uc.ns,
@@ -2121,7 +2217,7 @@ add_namespace_to_ts (void *cls, const char *name, const GNUNET_HashCode * id)
                                      PSEUDONYM_MC_NEXT_ID_EDITABLE, TRUE,
                                      PSEUDONYM_MC_CURRENT_ID_EDITABLE, TRUE,
                                      -1);
-  uc.seen = GNUNET_CONTAINER_multihashmap_create (128);
+  uc.seen = GNUNET_CONTAINER_multihashmap_create (128, GNUNET_NO);
   GNUNET_FS_namespace_list_updateable (uc.ns, NULL, &add_updateable_to_ts, &uc);
   GNUNET_CONTAINER_multihashmap_destroy (uc.seen);
 }
