@@ -873,6 +873,67 @@ preserve_meta_items (void *cls, const char *plugin_name,
 }
 
 
+/**
+ * Type of a function that libextractor calls for each
+ * meta data item found.  Used to get the mime type.
+ *
+ * @param cls pointer to where to store the mime type.
+ * @param plugin_name name of the plugin that produced this value;
+ *        special values can be used (i.e. '&lt;zlib&gt;' for zlib being
+ *        used in the main libextractor library and yielding
+ *        meta data).
+ * @param type libextractor-type describing the meta data
+ * @param format basic format information about data 
+ * @param data_mime_type mime-type of data (not of the original file);
+ *        can be NULL (if mime-type is not known)
+ * @param data actual meta-data found
+ * @param data_len number of bytes in data
+ * @return 0 to continue extracting, 1 to abort
+ */ 
+static int
+le_callback (void *cls,
+	     const char *plugin_name,
+	     enum EXTRACTOR_MetaType type,
+	     enum EXTRACTOR_MetaFormat format,
+	     const char *data_mime_type,
+	     const char *data,
+	     size_t data_len)
+{
+  char **ret = cls;
+
+  if ( (EXTRACTOR_METATYPE_MIMETYPE == type) &&
+       (EXTRACTOR_METAFORMAT_BINARY != format) )
+  {
+    *ret = GNUNET_strdup (data);
+    return 1;
+  }
+  return 0;
+}
+
+
+/**
+ * Use GNU libextractor to get the mime type for the given data.
+ *
+ * @param data binary data of the file
+ * @param data_size number of bytes in 'data'
+ */
+static char *
+get_mime_type (const void *data,
+	       gsize data_size)
+{
+  char *ret;
+  struct EXTRACTOR_PluginList *pl;
+
+  pl = EXTRACTOR_plugin_add (NULL, "mime", NULL, EXTRACTOR_OPTION_DEFAULT_POLICY);
+  if (NULL == pl)
+    return NULL;
+  ret = NULL;
+  EXTRACTOR_extract (pl, NULL, data, data_size, &le_callback, &ret);
+  EXTRACTOR_plugin_remove_all (pl);  
+  return ret;
+}
+
+
 /**
  * Function called to update the information in FI based on the changes made in
  * the edit dialog.
@@ -973,18 +1034,14 @@ file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
     gchar *fn;
     char *data;
     gsize data_size;
-    const char *mime;
+    char *mime;
     GFile *f;
-    GFileInfo *finfo;
 
     fn = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
                                         (gtk_builder_get_object
                                          (ctx->builder,
                                           "GNUNET_GTK_edit_publication_metadata_preview_file_chooser_button")));
     f = g_file_new_for_path (fn);
-    finfo =
-        g_file_query_info (f, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, 0, NULL,
-                           NULL);
     if (! g_file_load_contents (f, NULL, &data, &data_size, NULL, NULL))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -992,15 +1049,29 @@ file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
     }
     else
     {
-      mime =
-          g_file_info_get_attribute_string (finfo,
-                                            G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+      mime = get_mime_type (data, data_size);
+#if LINUX
+      if (NULL == mime)
+      {
+	/* fall back to Gtk mime-detection; this only works on GNU */
+	GFileInfo *finfo;
+	const char *gmime;
+	
+	finfo = g_file_query_info (f, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, 
+				   0, NULL, NULL);
+	gmime = g_file_info_get_attribute_string (finfo,
+						  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+	if (NULL != gmime)
+	  mime = GNUNET_strdup (gmime);
+	g_object_unref (finfo);
+      }
+#endif
       GNUNET_CONTAINER_meta_data_insert (meta, "<user>",
                                          EXTRACTOR_METATYPE_THUMBNAIL,
                                          EXTRACTOR_METAFORMAT_BINARY, mime,
                                          data, data_size);
+      GNUNET_free_non_null (mime);
     }
-    g_object_unref (finfo);
     g_object_unref (f);
     g_free (fn);
   }
