@@ -47,6 +47,13 @@
 #define GDK_KEY_Delete GDK_Delete
 #endif
 
+/**
+ * Handle for our main loop.
+ */
+struct GNUNET_GTK_MainLoop;
+
+
+
 /* ****************** Initialization *************** */
 
 /**
@@ -122,15 +129,18 @@ GNUNET_GTK_get_tree_string (GtkTreeView *treeview,
 			    GtkTreePath *treepath,
                             guint column, gchar ** value);
 
+
 /**
  * Create our tray icon.
  *
+ * @param ml main loop
  * @param main handle to the main window (show or hide)
  * @param icon_name name of the tray icon file
  * @param tooltip tooltip for the tray icon
  */
 void
-GNUNET_GTK_tray_icon_create (GtkWindow * main, const char *icon_name,
+GNUNET_GTK_tray_icon_create (struct GNUNET_GTK_MainLoop *ml,
+			     GtkWindow * main, const char *icon_name,
                              const char *tooltip);
 
 
@@ -174,11 +184,6 @@ GNUNET_FS_GTK_remove_treestore_subtree (GtkTreeStore * ts,
 
 /* ******************* main loop ***************** */
 
-/**
- * Handle for our main loop.
- */
-struct GNUNET_GTK_MainLoop;
-
 
 /**
  * Initialize the main loop.
@@ -220,6 +225,10 @@ GNUNET_GTK_main_loop_get_object (struct GNUNET_GTK_MainLoop *ml,
 GtkBuilder *
 GNUNET_GTK_main_loop_get_builder (struct GNUNET_GTK_MainLoop *ml);
 
+
+/**
+ * FIXME!
+ */
 int
 GNUNET_GTK_main_loop_build_window (struct GNUNET_GTK_MainLoop *ml,
                                    gpointer data);
@@ -254,6 +263,82 @@ void
 GNUNET_GTK_main_loop_quit (struct GNUNET_GTK_MainLoop *ml);
 
 
+/* ******************** animations ******************* */
+
+
+/**
+ * Handle for an animation.  Each animation file should only
+ * be loaded once and can then be used in multiple tree views.
+ */
+struct GNUNET_FS_AnimationContext;
+
+
+/**
+ * Handle to a tree view (and column) that contains animations.
+ * Whenever an animation is added to a tree view, this module
+ * must be told about the tree view and column for the animations
+ * to work.
+ */
+struct GNUNET_FS_AnimationTreeViewHandle;
+
+
+/**
+ * Create an animation context.  Usually animation contexts are
+ * created during the startup of the application and kept around
+ * until shutdown.
+ *
+ * @param filename name of the file with the animation to show
+ * @return context to use to get the animated pixbuf, NULL on error
+ */
+struct GNUNET_FS_AnimationContext *
+GNUNET_GTK_animation_context_create (const char *filename);
+
+
+/**
+ * Destroy an animation context.  Should only be called after the
+ * respective pixbufs are no longer needed.
+ *
+ * @param ac animation context to destroy.
+ */
+void
+GNUNET_GTK_animation_context_destroy (struct GNUNET_FS_AnimationContext *ac);
+
+
+/**
+ * Obtain the animated pixbuf from an animation context.  Note
+ * that the pixbuf will only properly work within GtkTreeViews
+ * where the column with the image has been registered using
+ * GNUNET_GTK_animation_tree_view_register.
+ *
+ * @param ac animation context to query
+ * @return pixbuf of the AC, NULL on error loading the pixbuf
+ */
+GdkPixbuf *
+GNUNET_GTK_animation_context_get_pixbuf (struct GNUNET_FS_AnimationContext *ac);
+
+
+/**
+ * Register a tree view column with the animation subsystem.  This
+ * column may contain animated pixbufs and thus needs to be periodically
+ * redrawn.
+ *
+ * @param tv tree view to register
+ * @param image_col column in the tree view with the animated pixbufs
+ * @return handle to unregister the tree view later
+ */
+struct GNUNET_FS_AnimationTreeViewHandle *
+GNUNET_GTK_animation_tree_view_register (GtkTreeView *tv,
+					 GtkTreeViewColumn *image_col);
+
+
+/**
+ * Unregister a tree view, it no longer needs to be updated.
+ *
+ * @param atv tree view to unregister
+ */
+void
+GNUNET_GTK_animation_tree_view_unregister (struct GNUNET_FS_AnimationTreeViewHandle *atv);
+
 
 #endif
 /* end of gnunet_gtk.h */
