@@ -190,6 +190,7 @@ GNUNET_GTK_main_loop_quit (struct GNUNET_GTK_MainLoop *ml)
   }
 }
 
+
 /**
  * Get the builder from the main window.
  *
@@ -428,7 +429,7 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
       {
         struct GNUNET_DISK_FileHandle *fh =
             GNUNET_CONTAINER_slist_get (&t, NULL);
-        if (fh->type == GNUNET_PIPE)
+        if (fh->type == GNUNET_DISK_HANLDE_TYPE_PIPE)
         {
           if (!ReadFile (fh->h, NULL, 0, NULL, fh->oOverlapRead))
           {
@@ -501,7 +502,7 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 
         DWORD dwBytes;
 
-        if (fh->type == GNUNET_PIPE)
+        if (fh->type == GNUNET_DISK_HANLDE_TYPE_PIPE)
         {
           if (!PeekNamedPipe (fh->h, NULL, 0, NULL, &dwBytes, NULL))
           {
@@ -770,7 +771,7 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
     {
       struct GNUNET_DISK_FileHandle *fh = GNUNET_CONTAINER_slist_get (&t, NULL);
 
-      if (fh->type == GNUNET_PIPE)
+      if (fh->type == GNUNET_DISK_HANLDE_TYPE_PIPE)
         CancelIo (fh->h);
     }
     GNUNET_NETWORK_fdset_zero (rfds);
@@ -827,6 +828,7 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   struct GNUNET_GTK_MainLoop *ml = cls;
   struct GNUNET_CONFIGURATION_Handle *gcfg;
   char *baseconfig;
+  char *ipath2;
   char *ipath;
   int argc;
 
@@ -837,16 +839,31 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   gtk_init (&argc, (char ***) &args);
   gcfg = GNUNET_CONFIGURATION_create ();
 
-  ipath = GNUNET_GTK_installation_get_path (GNUNET_OS_IPK_DATADIR);
-  if (ipath == NULL)
+  if (NULL == (ipath = GNUNET_OS_installation_get_path (GNUNET_OS_IPK_DATADIR)))
   {
     GNUNET_break (0);
     return;
   }
   GNUNET_asprintf (&baseconfig, "%s%s", ipath, "config.d");
-  GNUNET_free (ipath);
   (void) GNUNET_CONFIGURATION_load_from (gcfg, baseconfig);
   GNUNET_free (baseconfig);
+
+  if (NULL == (ipath2 = GNUNET_GTK_installation_get_path (GNUNET_OS_IPK_DATADIR)))
+  {
+    GNUNET_break (0);
+    return;
+  }
+  if (0 != strcmp (ipath, ipath2))
+  {
+    GNUNET_asprintf (&baseconfig, "%s%s", ipath, "config.d");
+    if (GNUNET_YES ==
+	GNUNET_DISK_directory_test (baseconfig, GNUNET_YES))
+      (void) GNUNET_CONFIGURATION_load_from (gcfg, baseconfig);
+    GNUNET_free (baseconfig);
+  }
+  GNUNET_free (ipath2);
+  GNUNET_free (ipath);
+
   (void) GNUNET_CONFIGURATION_load (gcfg, cfgfile);
   ml->rs = GNUNET_NETWORK_fdset_create ();
   ml->ws = GNUNET_NETWORK_fdset_create ();
@@ -921,8 +938,11 @@ GNUNET_GTK_main_loop_start (const char *binary_name, const char *binary_help,
     g_object_unref (G_OBJECT (ml.builder));
   if (NULL != ml.gml)
     g_main_loop_unref (ml.gml);
-  GNUNET_CONFIGURATION_destroy (ml.cfg);
-  ml.cfg = NULL;
+  if (NULL != ml.cfg)
+  {
+    GNUNET_CONFIGURATION_destroy (ml.cfg);
+    ml.cfg = NULL;
+  }
   GNUNET_free_non_null (ml.cfgfile);
   return ret;
 }
