@@ -184,7 +184,7 @@ get_object (const char *name)
  * @return GNUNET_OK (continue to iterate)
  */
 static int
-free_paths (void *cls, const GNUNET_HashCode * key, void *value)
+free_paths (void *cls, const struct GNUNET_HashCode * key, void *value)
 {
   struct PeerInfo *info = value;
 
@@ -213,8 +213,14 @@ free_paths (void *cls, const GNUNET_HashCode * key, void *value)
 static void
 shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
-  GNUNET_PEERINFO_notify_cancel (pnc);
-  pnc = NULL;
+  GNUNET_GTK_tray_icon_destroy ();
+  GNUNET_GTK_main_loop_quit (ml);
+  ml = NULL;
+  if (NULL != pnc)
+  {
+    GNUNET_PEERINFO_notify_cancel (pnc);
+    pnc = NULL;
+  }
   if (NULL != ats)
   {
     GNUNET_ATS_performance_done (ats);
@@ -282,7 +288,7 @@ peer_address_string_cb (void *cls, const char *address)
 		      PEERINFO_MC_CONNECTIVITY_LED, led_green,
 		      PEERINFO_MC_CONNECTED_STATUS, TRUE,
 		      -1);
-  GNUNET_free (country);
+  GNUNET_free_non_null (country);
   info->got_address = GNUNET_YES;
 }
 
@@ -433,9 +439,7 @@ status_cb (void *cls,
 void
 GNUNET_PEERINFO_GTK_quit_cb (GObject * object, gpointer user_data)
 {
-  GNUNET_GTK_tray_icon_destroy ();
-  GNUNET_GTK_main_loop_quit (ml);
-  GNUNET_SCHEDULER_add_now (&shutdown_task, NULL);
+  GNUNET_SCHEDULER_shutdown ();
 }
 
 
@@ -484,14 +488,15 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   led_red = load_led ("red");
   GNUNET_GTK_set_icon_search_path ();
   GNUNET_GTK_setup_nls ();
-  peer2info = GNUNET_CONTAINER_multihashmap_create (256);
+  peer2info = GNUNET_CONTAINER_multihashmap_create (256, GNUNET_NO);
   pnc =
       GNUNET_PEERINFO_notify (get_configuration (), &peerinfo_processor, NULL);
   if (pnc == NULL)
   {
     fprintf (stderr,
              _("Failed to initialize communication with peerinfo service!\n"));
-    exit (1);
+    GNUNET_SCHEDULER_shutdown ();
+    return;
   }
   ats = GNUNET_ATS_performance_init (get_configuration (), &status_cb, NULL);
   /* setup main window */
@@ -499,7 +504,8 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   ls = GTK_LIST_STORE (get_object ("GNUNET_PEERINFO_GTK_list_store"));
   GNUNET_assert (NULL != ls);
   gtk_window_maximize (GTK_WINDOW (main_window));
-  GNUNET_GTK_tray_icon_create (GTK_WINDOW (main_window),
+  GNUNET_GTK_tray_icon_create (ml,
+			       GTK_WINDOW (main_window),
                                "gnunet-gtk" /* FIXME: different icon? */ ,
                                "gnunet-peerinfo-gtk");
 
@@ -509,6 +515,8 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     gtk_widget_show (main_window);
     gtk_window_present (GTK_WINDOW (main_window));
   }
+  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
+				&shutdown_task, NULL);
 }
 
 
