@@ -357,11 +357,11 @@ static struct PortSpecification port_specifications[] = {
    gettext_noop ("the TCP transport plugin"),},
   {
    "GNUNET_setup_transport_http_port_spinbutton",
-   "GNUNET_setup_transport_http_checkbutton",
+   "GNUNET_setup_transport_http_server_checkbutton",
    gettext_noop ("the HTTP transport plugin"),},
   {
    "GNUNET_setup_transport_https_port_spinbutton",
-   "GNUNET_setup_transport_https_checkbutton",
+   "GNUNET_setup_transport_https_server_checkbutton",
    gettext_noop ("the HTTPS transport plugin"),},
   {NULL, NULL}
 };
@@ -716,660 +716,12 @@ save_string_list_store (const void *cls, const char *section,
 }
 
 
-/**
- * Check if the section represents a DNS entry and then update the
- * GtkListStore accordingly.
- *
- * @param cls the list store to modify
- * @param section name of the section
- */
-static void
-add_dns_entry_to_list_store (void *cls, const char *section)
-{
-  GtkListStore *ls = cls;
-  GtkTreeIter iter;
-  char *sld;
-  char *hostname;
-  char *hostport;
-  char *redirect;
-  char *cpy;
-  gboolean udp;
-
-  if (NULL == section)
-  {
-    gtk_list_store_insert_with_values (ls, &iter, G_MAXINT,
-                                       GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, "",
-                                       GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT,
-                                       (guint) 80,
-                                       GNUNET_GTK_SETUP_GNS_MC_TARGETPORT,
-                                       (guint) 8080,
-                                       GNUNET_GTK_SETUP_GNS_MC_HOSTNAME,
-                                       "localhost4",
-                                       GNUNET_GTK_SETUP_GNS_MC_ISUDP, "tcp",
-                                       -1);
-    return;
-  }
-
-  if ((8 > strlen (section)) ||
-      (0 != strcmp (".gnunet.", section + ((strlen (section) - 8)))))
-    return;
-  sld = GNUNET_strdup (section);
-  sld[strlen (section) - 8] = '\0';
-  udp = FALSE;
-  do
-  {
-    if (GNUNET_OK ==
-        GNUNET_CONFIGURATION_get_value_string (cfg, section,
-                                               (TRUE ==
-                                                udp) ? "UDP_REDIRECTS" :
-                                               "TCP_REDIRECTS", &cpy))
-    {
-      for (redirect = strtok (cpy, " "); redirect != NULL;
-           redirect = strtok (NULL, " "))
-      {
-        if (NULL == (hostname = strstr (redirect, ":")))
-        {
-          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                      _("Option `%s' is not formatted correctly!\n"), redirect);
-          continue;
-        }
-        hostname[0] = '\0';
-        hostname++;
-        if (NULL == (hostport = strstr (hostname, ":")))
-        {
-          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                      _("Option `%s' is not formatted correctly!\n"), redirect);
-          continue;
-        }
-        hostport[0] = '\0';
-        hostport++;
-
-        int local_port = atoi (redirect);
-
-        if (!((local_port > 0) && (local_port < 65536)))
-        {
-          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                      _("`%s' is not a valid port number!\n"), redirect);
-          continue;
-        }
-        int host_port = atoi (hostport);
-
-        if (!((host_port > 0) && (host_port < 65536)))
-        {
-          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                      _("`%s' is not a valid port number!\n"), hostport);
-          continue;
-        }
-        gtk_list_store_insert_with_values (ls, &iter, 0,
-                                           GNUNET_GTK_SETUP_GNS_MC_HOSTNAME,
-                                           sld,
-                                           GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT,
-                                           (guint) local_port,
-                                           GNUNET_GTK_SETUP_GNS_MC_TARGETPORT,
-                                           (guint) host_port,
-                                           GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME,
-                                           hostname,
-                                           GNUNET_GTK_SETUP_GNS_MC_ISUDP,
-                                           (TRUE == udp) ? "udp" : "tcp", -1);
-      }
-      GNUNET_free (cpy);
-    }
-    udp = !udp;
-  }
-  while (udp != FALSE);
-  GNUNET_free (sld);
-}
-
-
-/**
- * Initialize the GtkListModel with the VPN's DNS service specification.
- *
- * @param cls NULL
- * @param section section with the value (NULL)
- * @param option option name (NULL)
- * @param value value as a string (NULL)
- * @param widget widget to initialize (the GtkTreeView)
- * @param cfg configuration handle
- * @return GNUNET_OK on success, GNUNET_SYSERR if there was a problem
- */
-static int
-load_vpn_dns_configuration (const void *cls, const char *section,
-                            const char *option, const char *value,
-                            GObject * widget,
-                            const struct GNUNET_CONFIGURATION_Handle *cfg)
-{
-  GtkTreeView *tv;
-  GtkListStore *ls;
-
-  tv = GTK_TREE_VIEW (widget);
-  if (tv == NULL)
-  {
-    GNUNET_break (0);
-    return GNUNET_SYSERR;
-  }
-  ls = GTK_LIST_STORE (gtk_tree_view_get_model (tv));
-  GNUNET_CONFIGURATION_iterate_sections (cfg, &add_dns_entry_to_list_store, ls);
-  /* finally, add empty entry */
-  add_dns_entry_to_list_store (ls, NULL);
-  return GNUNET_OK;
-}
-
-
-/**
- * Records we use to build DNS information lists.
- */
-struct DnsInfo
-{
-  struct DnsInfo *next;
-  char *section;
-  char *altnames;
-  char *tcpred;
-  char *udpred;
-  unsigned long long ttl;
-};
-
-
-/**
- * Function called for each section in the configuration.
- * Gather existing ttl, section names and altnames.
- *
- * @param cls 'struct DnsInfo**' to create
- * @param section name of a section in the configuration
- */
-static void
-collect_dns_sections (void *cls, const char *section)
-{
-  struct DnsInfo **base = cls;
-  struct DnsInfo *pos;
-
-  if ((8 > strlen (section)) ||
-      (0 != strcmp (".gnunet.", section + ((strlen (section) - 8)))))
-    return;
-  pos = GNUNET_malloc (sizeof (struct DnsInfo));
-  pos->section = GNUNET_strdup (section);
-  if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_number (cfg, section, "TTL", &pos->ttl))
-    pos->ttl = 3600000;
-  if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_string (cfg, section, "ALTERNATIVE_NAMES",
-                                             &pos->altnames))
-    pos->altnames = NULL;
-  pos->tcpred = GNUNET_strdup ("");
-  pos->udpred = GNUNET_strdup ("");
-  pos->next = *base;
-  *base = pos;
-}
-
-
-/**
- * Function called for each section in the configuration.
- * Removes those ending in '.gnunet.'.
- *
- * @param cls unused
- * @param section name of a section in the configuration
- */
-static void
-remove_dns_sections (void *cls, const char *section)
-{
-  if ((8 > strlen (section)) ||
-      (0 != strcmp (".gnunet.", section + ((strlen (section) - 8)))))
-    return;
-  GNUNET_CONFIGURATION_remove_section (cfg, section);
-}
-
-
-/**
- * Given the list store and the data in it, update the
- * configuration file accordingly.
- *
- * @param tm model to use
- */
-static void
-update_vpn_dns_configuration (GtkTreeModel * tm)
-{
-  GtkTreeIter iter;
-  gchar *hostname;
-  guint srcport;
-  guint targetport;
-  gchar *targethost;
-  gchar *tcpudp;
-  char *tmp;
-  struct DnsInfo *head;
-  struct DnsInfo *pos;
-
-  head = NULL;
-  GNUNET_CONFIGURATION_iterate_sections (cfg, &collect_dns_sections, &head);
-  if (TRUE == gtk_tree_model_get_iter_first (tm, &iter))
-    do
-    {
-      gtk_tree_model_get (tm, &iter,
-                          GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, &hostname,
-                          GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT, &srcport,
-                          GNUNET_GTK_SETUP_GNS_MC_TARGETPORT, &targetport,
-                          GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME, &targethost,
-                          GNUNET_GTK_SETUP_GNS_MC_ISUDP, &tcpudp,
-                          -1);
-      if (0 != strlen (hostname))
-      {
-        pos = head;
-        GNUNET_asprintf (&tmp, "%s.gnunet.", hostname);
-        while ((NULL != pos) && (0 != strcasecmp (tmp, pos->section)))
-          pos = pos->next;
-        if (pos == NULL)
-        {
-          pos = GNUNET_malloc (sizeof (struct DnsInfo));
-          pos->section = tmp;
-          pos->ttl = 3600000;
-          pos->altnames = NULL;
-          pos->tcpred = GNUNET_strdup ("");
-          pos->udpred = GNUNET_strdup ("");
-          pos->next = head;
-          head = pos;
-        }
-        else
-        {
-          GNUNET_free (tmp);
-        }
-        GNUNET_asprintf (&tmp, "%u:%s:%u %s", srcport, targethost, targetport,
-                         (0 ==
-                          strcasecmp ("tcp",
-                                      tcpudp)) ? pos->tcpred : pos->udpred);
-        if (0 == strcasecmp ("tcp", tcpudp))
-        {
-          GNUNET_free (pos->tcpred);
-          pos->tcpred = tmp;
-        }
-        else
-        {
-          GNUNET_free (pos->udpred);
-          pos->udpred = tmp;
-        }
-      }
-      g_free (tcpudp);
-      g_free (hostname);
-      g_free (targethost);
-    }
-    while (TRUE == gtk_tree_model_iter_next (tm, &iter));
-  GNUNET_CONFIGURATION_iterate_sections (cfg, &remove_dns_sections, NULL);
-  while (NULL != head)
-  {
-    pos = head;
-    head = pos->next;
-    if (pos->altnames != NULL)
-      GNUNET_CONFIGURATION_set_value_string (cfg, pos->section,
-                                             "ALTERNATIVE_NAMES",
-                                             pos->altnames);
-    if (strlen (pos->udpred) > 0)
-      GNUNET_CONFIGURATION_set_value_string (cfg, pos->section, "UDP_REDIRECTS",
-                                             pos->udpred);
-    if (strlen (pos->tcpred) > 0)
-      GNUNET_CONFIGURATION_set_value_string (cfg, pos->section, "TCP_REDIRECTS",
-                                             pos->tcpred);
-    if ((strlen (pos->udpred) > 0) || (strlen (pos->tcpred) > 0))
-      GNUNET_CONFIGURATION_set_value_number (cfg, pos->section, "TTL",
-                                             pos->ttl);
-    GNUNET_free_non_null (pos->altnames);
-    GNUNET_free (pos->tcpred);
-    GNUNET_free (pos->udpred);
-    GNUNET_free (pos->section);
-    GNUNET_free (pos);
-  }
-}
-
-
-/**
- * The user has edited the DNS name of a service we're offering.
- * Update the GtkTreeModel (at the given path) and update the
- * respective service entry in the configuration file.  Finally,
- * if the edited path is for a "fresh" entry, create another empty
- * one at the bottom.  If the hostname was set to empty, remove
- * the entire entry from the configuration and the model.
- *
- * @param renderer GtkCellRendererText that changed
- * @param path GtkTreePath identifying where in the Model the change is
- * @param new_text the new text that was stored in the line
- * @param user_data NULL
- */
-static void
-save_vpn_dns_service_dnsname (GtkCellRendererText * renderer, gchar * path,
-                              gchar * new_text, gpointer user_data)
-{
-  GtkTreeModel *tm;
-  GtkListStore *ls;
-  GtkTreeIter iter;
-  gchar *old;
-
-  tm = GTK_TREE_MODEL (GNUNET_SETUP_get_object ("vpn_dns_config_liststore"));
-  if (NULL == tm)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  ls = GTK_LIST_STORE (tm);
-  if (NULL == ls)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  if (TRUE != gtk_tree_model_get_iter_from_string (tm, &iter, path))
-  {
-    GNUNET_break (0);
-    return;
-  }
-  gtk_tree_model_get (tm, &iter, GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, &old, -1);
-  if ((0 != strlen (old)) && (0 == strlen (new_text)))
-  {
-    /* deletion */
-    gtk_list_store_remove (ls, &iter);
-    g_free (old);
-    /* update configuration */
-    update_vpn_dns_configuration (tm);
-    return;
-  }
-  /* update model */
-  gtk_list_store_set (ls, &iter,
-                      GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, new_text, -1);
-  /* update configuration */
-  update_vpn_dns_configuration (tm);
-  if ((0 == strlen (old)) && (0 != strlen (new_text)))
-  {
-    /* need another empty entry at the end for future expansion */
-    add_dns_entry_to_list_store (GTK_LIST_STORE (tm), NULL);
-  }
-  g_free (old);
-}
-
-
-/**
- * Initialize the GtkListModel with the VPN's DNS service specification.
- *
- * @param cls NULL
- * @param section section with the value (NULL)
- * @param option option name (NULL)
- * @param widget widget to initialize (the GtkTreeView)
- * @param cfg configuration handle
- * @return GNUNET_OK on success, GNUNET_SYSERR if there was a problem
- */
-static int
-gns_name_install_edited_handler (const void *cls,
-				 const char *section,
-				 const char *option,
-				 GObject * widget,
-				 struct
-				 GNUNET_CONFIGURATION_Handle
-				 *cfg)
-{
-  static int once;
-  GtkCellRendererText *rt;
-
-  rt = GTK_CELL_RENDERER_TEXT (widget);
-  if (NULL == rt)
-    return GNUNET_SYSERR;
-  if (0 != once++)
-    return GNUNET_OK;
-  g_signal_connect (rt, "edited", G_CALLBACK (&save_vpn_dns_service_dnsname),
-                    NULL);
-  return GNUNET_OK;
-}
-
-
-/**
- * The user has edited the DNS name of a service we're offering.
- * Update the GtkTreeModel (at the given path) and update the
- * respective service entry in the configuration file.  Finally,
- * if the edited path is for a "fresh" entry, create another empty
- * one at the bottom.  If the hostname was set to empty, remove
- * the entire entry from the configuration and the model.
- *
- * @param renderer GtkCellRendererText that changed
- * @param path GtkTreePath identifying where in the Model the change is
- * @param new_text the new text that was stored in the line
- * @param user_data NULL
- */
-static void
-save_vpn_dns_service_tcpudp (GtkCellRendererText * renderer, gchar * path,
-                             gchar * new_text, gpointer user_data)
-{
-  GtkTreeModel *tm;
-  GtkListStore *ls;
-  GtkTreeIter iter;
-
-  if ((0 != strcasecmp ("tcp", new_text)) &&
-      (0 != strcasecmp ("udp", new_text)))
-  {
-    /* FIXME: warn... */
-    return;
-  }
-  tm = GTK_TREE_MODEL (GNUNET_SETUP_get_object ("vpn_dns_config_liststore"));
-  if (NULL == tm)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  ls = GTK_LIST_STORE (tm);
-  if (NULL == ls)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  if (TRUE != gtk_tree_model_get_iter_from_string (tm, &iter, path))
-  {
-    GNUNET_break (0);
-    return;
-  }
-  /* update model */
-  gtk_list_store_set (ls, &iter, GNUNET_GTK_SETUP_GNS_MC_ISUDP, new_text, -1);
-  /* update configuration */
-  update_vpn_dns_configuration (tm);
-}
-
-
-/**
- * Initialize the GtkListModel with the VPN's DNS service specification.
- *
- * @param cls NULL
- * @param section section with the value (NULL)
- * @param option option name (NULL)
- * @param widget widget to initialize (the GtkTreeView)
- * @param cfg configuration handle
- * @return GNUNET_OK on success, GNUNET_SYSERR if there was a problem
- */
-static int
-gns_type_install_edited_handler (const void *cls,
-				 const char *section,
-				 const char *option,
-				 GObject * widget,
-				 struct
-                                               GNUNET_CONFIGURATION_Handle *cfg)
-{
-  static int once;
-  GtkCellRendererText *rt;
-
-  rt = GTK_CELL_RENDERER_TEXT (widget);
-  if (NULL == rt)
-    return GNUNET_SYSERR;
-  if (0 != once++)
-    return GNUNET_OK;
-  g_signal_connect (rt, "edited", G_CALLBACK (&save_vpn_dns_service_tcpudp),
-                    NULL);
-  return GNUNET_OK;
-}
-
-
-/**
- * The user has edited the DNS name of a service we're offering.
- * Update the GtkTreeModel (at the given path) and update the
- * respective service entry in the configuration file.  Finally,
- * if the edited path is for a "fresh" entry, create another empty
- * one at the bottom.  If the hostname was set to empty, remove
- * the entire entry from the configuration and the model.
- *
- * @param renderer GtkCellRendererText that changed
- * @param path GtkTreePath identifying where in the Model the change is
- * @param new_text the new text that was stored in the line
- * @param user_data NULL
- */
-static void
-save_vpn_dns_service_sourceport (GtkCellRendererText * renderer, gchar * path,
-                                 gchar * new_text, gpointer user_data)
-{
-  GtkTreeModel *tm;
-  GtkListStore *ls;
-  GtkTreeIter iter;
-  int port;
-
-  port = atoi (new_text);
-  if ((port < 1) || (port > UINT16_MAX))
-  {
-    /* invalid port, FIXME: warn */
-    return;
-  }
-  tm = GTK_TREE_MODEL (GNUNET_SETUP_get_object ("vpn_dns_config_liststore"));
-  if (NULL == tm)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  ls = GTK_LIST_STORE (tm);
-  if (NULL == ls)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  if (TRUE != gtk_tree_model_get_iter_from_string (tm, &iter, path))
-  {
-    GNUNET_break (0);
-    return;
-  }
-  /* update model */
-  gtk_list_store_set (ls, &iter,
-                      GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT, (guint) port, -1);
-  /* update configuration */
-  update_vpn_dns_configuration (tm);
-}
-
-
-/**
- * Initialize the GtkListModel with the VPN's DNS service specification.
- *
- * @param cls NULL
- * @param section section with the value (NULL)
- * @param option option name (NULL)
- * @param widget widget to initialize (the GtkTreeView)
- * @param cfg configuration handle
- * @return GNUNET_OK on success, GNUNET_SYSERR if there was a problem
- */
-static int
-gns_ttl_install_edited_handler (const void *cls,
-				const char *section,
-				const char *option,
-				GObject * widget,
-				struct
-				GNUNET_CONFIGURATION_Handle
-				*cfg)
-{
-  static int once;
-  GtkCellRendererText *rt;
-
-  rt = GTK_CELL_RENDERER_TEXT (widget);
-  if (NULL == rt)
-    return GNUNET_SYSERR;
-  if (0 != once++)
-    return GNUNET_OK;
-  g_signal_connect (rt, "edited", G_CALLBACK (&save_vpn_dns_service_sourceport),
-                    NULL);
-  return GNUNET_OK;
-}
-
-
-/**
- * The user has edited the DNS name of a service we're offering.
- * Update the GtkTreeModel (at the given path) and update the
- * respective service entry in the configuration file.  Finally,
- * if the edited path is for a "fresh" entry, create another empty
- * one at the bottom.  If the hostname was set to empty, remove
- * the entire entry from the configuration and the model.
- *
- * @param renderer GtkCellRendererText that changed
- * @param path GtkTreePath identifying where in the Model the change is
- * @param new_text the new text that was stored in the line
- * @param user_data NULL
- */
-static void
-save_vpn_dns_service_targethostname (GtkCellRendererText * renderer,
-                                     gchar * path, gchar * new_text,
-                                     gpointer user_data)
-{
-  GtkTreeModel *tm;
-  GtkListStore *ls;
-  GtkTreeIter iter;
-
-  tm = GTK_TREE_MODEL (GNUNET_SETUP_get_object ("vpn_dns_config_liststore"));
-  if (NULL == tm)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  ls = GTK_LIST_STORE (tm);
-  if (NULL == ls)
-  {
-    GNUNET_break (0);
-    return;
-  }
-  if (TRUE != gtk_tree_model_get_iter_from_string (tm, &iter, path))
-  {
-    GNUNET_break (0);
-    return;
-  }
-  /* update model */
-  gtk_list_store_set (ls, &iter,
-                      GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME, new_text, -1);
-  /* update configuration */
-  update_vpn_dns_configuration (tm);
-}
-
-
-/**
- * Initialize the GtkListModel with the VPN's DNS service specification.
- *
- * @param cls NULL
- * @param section section with the value (NULL)
- * @param option option name (NULL)
- * @param widget widget to initialize (the GtkTreeView)
- * @param cfg configuration handle
- * @return GNUNET_OK on success, GNUNET_SYSERR if there was a problem
- */
-static int
-gns_value_install_edited_handler (const void *cls,
-				  const char *section,
-				  const char *option,
-				  GObject * widget,
-				  struct
-				  GNUNET_CONFIGURATION_Handle
-				  *cfg)
-{
-  static int once;
-  GtkCellRendererText *rt;
-
-  rt = GTK_CELL_RENDERER_TEXT (widget);
-  if (NULL == rt)
-    return GNUNET_SYSERR;
-  if (0 != once++)
-    return GNUNET_OK;
-  g_signal_connect (rt, "edited",
-                    G_CALLBACK (&save_vpn_dns_service_targethostname), NULL);
-  return GNUNET_OK;
-}
-
-
-
 /**
  * Hide "min connected friends" option if in F2F-only mode.
  */
-static struct GNUNET_SETUP_VisibilitySpecification hide_min_connected_friends[]
-    = {
-  {"GNUNET_setup_minimum_friends_label", NULL, REX_YES},
-  {"GNUNET_setup_minimum_friends_spinbutton", NULL, REX_YES},
+static struct GNUNET_SETUP_VisibilitySpecification hide_min_connected_friends[] = {
+  {"GNUNET_setup_minimum_friends_label", NULL, REX_YES },
+  {"GNUNET_setup_minimum_friends_spinbutton", NULL, REX_YES },
   {NULL, NULL, NULL}
 };
 
@@ -1411,7 +763,7 @@ static struct GNUNET_SETUP_VisibilitySpecification
 
 
 /**
- * Hide "fs tab" if FS not active.
+ * Hide "fs" tab if FS not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_fs_tab[] = {
   {"GNUNET_setup_fs_main_vbox", "(^| )fs($| )", NULL},
@@ -1420,7 +772,7 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_fs_tab[] = {
 
 
 /**
- * Hide "vpn tab" if VPN not active.
+ * Hide "vpn" tab if VPN not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_vpn_tab[] = {
   {"GNUNET_setup_vpn_vbox", "(^| )pt($| )", NULL},
@@ -1429,16 +781,17 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_vpn_tab[] = {
 
 
 /**
- * Hide "gns tab" if GNS not active.
+ * Hide "gads" and "namestore" tabs if GNS not active.
  */
-static struct GNUNET_SETUP_VisibilitySpecification hide_gns_tab[] = {
-  {"GNUNET_setup_gns_vbox", "(^| )gns($| )", NULL},
+static struct GNUNET_SETUP_VisibilitySpecification hide_gns_tabs[] = {
+  {"GNUNET_setup_gads_vbox", "(^| )gns($| )", NULL},
+  {"GNUNET_setup_namestore_vbox", "(^| )gns($| )", NULL},
   {NULL, NULL, NULL}
 };
 
 
 /**
- * Hide "tcp tab" if TCP not active.
+ * Hide "tcp" tab if TCP not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_tcp_tab[] = {
   {"GNUNET_setup_transport_tcp_vbox", "(^| )tcp($| )", NULL},
@@ -1447,7 +800,7 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_tcp_tab[] = {
 
 
 /**
- * Hide "udp tab" if UDP not active.
+ * Hide "udp" tab if UDP not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_udp_tab[] = {
   {"GNUNET_setup_transport_udp_vbox", "(^| )udp($| )", NULL},
@@ -1456,25 +809,25 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_udp_tab[] = {
 
 
 /**
- * Hide "http tab" if HTTP not active.
+ * Hide "http" tab if HTTP not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_http_tab[] = {
-  {"GNUNET_setup_transport_http_vbox", "(^| )http($| )", NULL},
+  {"GNUNET_setup_transport_http_vbox", "(^| )http_server($| )", NULL},
   {NULL, NULL, NULL}
 };
 
 
 /**
- * Hide "https tab" if HTTPS not active.
+ * Hide "https" tab if HTTPS not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_https_tab[] = {
-  {"GNUNET_setup_transport_https_vbox", "(^| )https($| )", NULL},
+  {"GNUNET_setup_transport_https_vbox", "(^| )https_server($| )", NULL},
   {NULL, NULL, NULL}
 };
 
 
 /**
- * Hide "dv tab" if DV not active.
+ * Hide "dv" tab if DV not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_dv_tab[] = {
   {"GNUNET_setup_transport_dv_vbox", "(^| )dv($| )", NULL},
@@ -1483,7 +836,7 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_dv_tab[] = {
 
 
 /**
- * Hide "wlan tab" if WLAN not active.
+ * Hide "wlan" tab if WLAN not active.
  */
 static struct GNUNET_SETUP_VisibilitySpecification hide_wlan_tab[] = {
   {"GNUNET_setup_transport_wlan_vbox", "(^| )wlan($| )", NULL},
@@ -1529,10 +882,10 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_sqlite_datacache_tab[] =
 
 
 /**
- * Hide "mysql datacache" tab if mysql not active.
+ * Hide "heap datacache" tab if heap not active.
  */
-static struct GNUNET_SETUP_VisibilitySpecification hide_mysql_datacache_tab[] = {
-  {"GNUNET_setup_datacache_mysql_vbox", "^mysql$", NULL},
+static struct GNUNET_SETUP_VisibilitySpecification hide_heap_datacache_tab[] = {
+  {"GNUNET_setup_datacache_heap_vbox", "^heap$", NULL},
   {NULL, NULL, NULL}
 };
 
@@ -1547,6 +900,24 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_postgres_datacache_tab[]
 };
 
 
+/**
+ * Hide "sqlite namestore" tab if sqlite not active.
+ */
+static struct GNUNET_SETUP_VisibilitySpecification hide_sqlite_namestore_tab[] = {
+  {"GNUNET_setup_namestore_sqlite_label", "^sqlite$", NULL},
+  {NULL, NULL, NULL}
+};
+
+
+/**
+ * Hide "postgres datastore" tab if postgres not active.
+ */
+static struct GNUNET_SETUP_VisibilitySpecification hide_postgres_namestore_tab[] = {
+  {"GNUNET_setup_namestore_postgres_hbox", "^postgres$", NULL},
+  {NULL, NULL, NULL}
+};
+
+
 /**
  * Hide advertised TCP port if port is zero.
  */
@@ -1810,29 +1181,53 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    hide_udp_tab},
 
   {
-   "GNUNET_setup_transport_http_checkbutton",
+   "GNUNET_setup_transport_http_server_checkbutton",
    "toggled",
    "transport",
    "PLUGINS",
-   gettext_noop ("Enable communication via HTTP"),
+   gettext_noop ("Enable communication via HTTP as a server"),
    "https://gnunet.org/configuration-http",
    &load_option_list,
-   &save_option_list, "http",
+   &save_option_list, "http_server",
    &highlight_port_collisions, NULL,
    hide_http_tab},
 
   {
-   "GNUNET_setup_transport_https_checkbutton",
+   "GNUNET_setup_transport_http_client_checkbutton",
    "toggled",
    "transport",
    "PLUGINS",
-   gettext_noop ("Enable communication via HTTPS"),
+   gettext_noop ("Enable communication via HTTP as a client"),
+   "https://gnunet.org/configuration-http",
+   &load_option_list,
+   &save_option_list, "http_client",
+   &highlight_port_collisions, NULL,
+   NULL},
+
+  {
+   "GNUNET_setup_transport_https_server_checkbutton",
+   "toggled",
+   "transport",
+   "PLUGINS",
+   gettext_noop ("Enable communication via HTTPS as a server"),
    "https://gnunet.org/configuration-https",
    &load_option_list,
-   &save_option_list, "https",
+   &save_option_list, "https_server",
    &highlight_port_collisions, NULL,
    hide_https_tab},
 
+  {
+   "GNUNET_setup_transport_https_client_checkbutton",
+   "toggled",
+   "transport",
+   "PLUGINS",
+   gettext_noop ("Enable communication via HTTPS as a client"),
+   "https://gnunet.org/configuration-https",
+   &load_option_list,
+   &save_option_list, "https_client",
+   &highlight_port_collisions, NULL,
+   NULL},
+
   {
    "GNUNET_setup_transport_dv_checkbutton",
    "toggled",
@@ -1991,11 +1386,23 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    &save_number, NULL,
    NULL, NULL,
    NULL},
+   
+  {
+   "GNUNET_setup_transport_udp_advertised_port_spinbutton",
+   "value-changed",
+   "transport-udp",
+   "ADVERTISED_PORT",
+   gettext_noop ("Port visible to other peers"),
+   "https://gnunet.org/configuration-udp",
+   &load_number,
+   &save_number, NULL,
+   NULL, NULL,
+   NULL},
 
   {
    "GNUNET_setup_transport_http_port_spinbutton",
    "value-changed",
-   "transport-http",
+   "transport-http_server",
    "PORT",
    gettext_noop ("Port for inbound HTTP connections, use 0 if behind NAT"),
    "https://gnunet.org/configuration-http",
@@ -2004,10 +1411,22 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    &highlight_port_collisions, NULL,
    NULL},
 
+  {
+   "GNUNET_setup_transport_http_advertised_port_spinbutton",
+   "value-changed",
+   "transport-http_server",
+   "ADVERTISED_PORT",
+   gettext_noop ("Port visible to other peers"),
+   "https://gnunet.org/configuration-http",
+   &load_number,
+   &save_number, NULL,
+   NULL, NULL,
+   NULL},
+
   {
    "GNUNET_setup_transport_https_port_spinbutton",
    "value-changed",
-   "transport-https",
+   "transport-https_server",
    "PORT",
    gettext_noop ("Port for inbound HTTPS connections, use 0 if behind NAT"),
    "https://gnunet.org/configuration-https",
@@ -2016,8 +1435,32 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    &highlight_port_collisions, NULL,
    NULL},
 
+  {
+   "GNUNET_setup_transport_https_advertised_port_spinbutton",
+   "value-changed",
+   "transport-https_server",
+   "ADVERTISED_PORT",
+   gettext_noop ("Port visible to other peers"),
+   "https://gnunet.org/configuration-https",
+   &load_number,
+   &save_number, NULL,
+   NULL, NULL,
+   NULL},
+
   /* FS TAB */
 
+  {
+   "GNUNET_setup_fs_datastore_quota_spinbutton",
+   "value-changed",
+   "datastore",
+   "QUOTA",
+   gettext_noop ("How many bytes are we allowed to store in the local datastore?"),
+   "https://gnunet.org/datastore-quota",
+   &load_number,
+   &save_number, NULL,
+   NULL, NULL,
+   NULL},
+
   {
    "GNUNET_setup_fs_datastore_sqlite_radiobutton",
    "toggled",
@@ -2163,6 +1606,20 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    NULL},
 
+
+  {
+   "GNUNET_setup_fs_datacache_quota_spinbutton",
+   "value-changed",
+   "dhtcache",
+   "QUOTA",
+   gettext_noop ("How many bytes are we allowed to store in the local datacache?"),
+   "https://gnunet.org/datacache-quota",
+   &load_number,
+   &save_number, NULL,
+   NULL, NULL,
+   NULL},
+
+
   {
    "GNUNET_setup_fs_datacache_sqlite_radiobutton",
    "toggled",
@@ -2176,16 +1633,16 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    hide_sqlite_datacache_tab},
 
   {
-   "GNUNET_setup_fs_datacache_mysql_radiobutton",
+   "GNUNET_setup_fs_datacache_heap_radiobutton",
    "toggled",
    "dhtcache",
    "DATABASE",
-   gettext_noop ("Use MySQL to cache DHT data"),
+   gettext_noop ("Use memory to cache DHT data"),
    "https://gnunet.org/configuration-datacache",
    &load_option_list /* abuse! */ ,
-   &save_option_list /* abuse! */ , "mysql",
+   &save_option_list /* abuse! */ , "heap",
    NULL, NULL,
-   hide_mysql_datacache_tab},
+   hide_heap_datacache_tab},
 
   {
    "GNUNET_setup_fs_datacache_postgres_radiobutton",
@@ -2199,66 +1656,6 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    hide_postgres_datacache_tab},
 
-  {
-   "GNUNET_setup_datacache_mysql_database_name_entry",
-   "changed",
-   "datacache-mysql",
-   "DATABASE",
-   gettext_noop ("Name for the MySQL database"),
-   "https://gnunet.org/configuration-datacache",
-   &load_text,
-   &save_text, NULL,
-   NULL, NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_datacache_mysql_config_file_filechooserbutton",
-   "selection-changed",
-   "datacache-mysql",
-   "CONFIG",
-   gettext_noop ("Configuration file for MySQL access"),
-   "http://dev.mysql.com/doc/refman/5.5/en/option-files.html",
-   &load_filename,
-   &save_filename, NULL,
-   NULL, NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_datacache_mysql_username_entry",
-   "changed",
-   "datacache-mysql",
-   "USER",
-   gettext_noop ("Username for MySQL access"),
-   "https://gnunet.org/configuration-datacache",
-   &load_text,
-   &save_text, NULL,
-   NULL, NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_datacache_mysql_password_entry",
-   "changed",
-   "datacache-mysql",
-   "PASSWORD",
-   gettext_noop ("Password for MySQL access"),
-   "https://gnunet.org/configuration-datacache",
-   &load_text,
-   &save_text, NULL,
-   NULL, NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_datacache_mysql_hostname_entry",
-   "changed",
-   "datacache-mysql",
-   "HOST",
-   gettext_noop ("Name of host running MySQL database"),
-   "https://gnunet.org/configuration-datacache",
-   &load_text,
-   &save_text, NULL,
-   NULL, NULL,
-   NULL},
-
   {
    "GNUNET_setup_transport_wlan_interface_entry",
    "changed",
@@ -2271,18 +1668,6 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    NULL},
 
-  {
-   "GNUNET_setup_datacache_mysql_port_spinbutton",
-   "value-changed",
-   "datacache-mysql",
-   "PORT",
-   gettext_noop ("Port of MySQL database"),
-   "https://gnunet.org/configuration-datacache",
-   &load_number,
-   &save_number, NULL,
-   NULL, NULL,
-   NULL},
-
   {
    "GNUNET_setup_datacache_postgres_config_entry",
    "changed",
@@ -2541,7 +1926,47 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    NULL},
 
-  /* GNS treeview */
+
+  /* Namestore TAB */
+
+  {
+   "GNUNET_setup_namestore_sqlite_radiobutton",
+   "toggled",
+   "namestore",
+   "DATABASE",
+   gettext_noop ("Use sqLite to store names"),
+   "https://gnunet.org/configuration-namestore",
+   &load_option_list /* abuse! */ ,
+   &save_option_list /* abuse! */ , "sqlite",
+   NULL, NULL,
+   hide_sqlite_namestore_tab},
+
+  {
+   "GNUNET_setup_namestore_postgres_radiobutton",
+   "toggled",
+   "namestore",
+   "DATABASE",
+   gettext_noop ("Use PostGres to store names"),
+   "https://gnunet.org/configuration-namestore",
+   &load_option_list /* abuse! */ ,
+   &save_option_list /* abuse! */ , "postgres",
+   NULL, NULL,
+   hide_postgres_namestore_tab},
+
+  {
+   "GNUNET_setup_namestore_postgres_config_entry",
+   "changed",
+   "namestore-postgres",
+   "CONFIG",
+   gettext_noop ("Configuration for Postgres (passed to PQconnectdb)"),
+   "http://www.postgresql.org/docs/8.1/static/libpq.html#LIBPQ-CONNECT",
+   &load_text,
+   &save_text, NULL,
+   NULL, NULL,
+   NULL},
+
+
+  /* GADS/GNS */
 
   {
    "GNUNET_setup_general_services_gns_checkbutton",
@@ -2553,75 +1978,31 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    &load_option_list,
    &save_option_list, "gns",
    NULL, NULL,
-   hide_gns_tab},
+   hide_gns_tabs},
+
 
   {
-   "GNUNET_setup_gns_treeview",
-   NULL,
-   NULL,
-   NULL,
-   gettext_noop
-   ("Specification of .gnunet TLD"),
-   "https://gnunet.org/configuration-dns",
-   &load_vpn_dns_configuration,
-   NULL, NULL,
+   "GNUNET_setup_gns_hijack_checkbutton",
+   "toggled",
+   "gns",
+   "HIJACK_DNS",
+   gettext_noop ("Should DNS queries be intercepted using iptables?"),
+   "https://gnunet.org/gnunet-dns-setup",
+   &load_yes_no,
+   &save_yes_no, NULL,
    NULL, NULL,
    NULL},
 
   {
-   "GNUNET_setup_gns_name_cellrenderertext",
-   "editing-started",
-   NULL,
-   NULL,
-   NULL,
-   "https://gnunet.org/configuration-dns",
-   NULL,
-   &gns_name_install_edited_handler,
-   NULL,
-   NULL,
-   NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_gns_type_cellrenderertext",
-   "editing-started",
-   NULL,
-   NULL,
-   NULL,
-   "https://gnunet.org/configuration-dns",
-   NULL,
-   &gns_type_install_edited_handler,
-   NULL,
-   NULL,
-   NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_gns_ttl_cellrenderertext",
-   "editing-started",
-   NULL,
-   NULL,
-   NULL,
-   "https://gnunet.org/configuration-dns",
-   NULL,
-   &gns_ttl_install_edited_handler,
-   NULL,
-   NULL,
-   NULL,
-   NULL},
-
-  {
-   "GNUNET_setup_gns_value_cellrenderertext",
-   "editing-started",
-   NULL,
-   NULL,
-   NULL,
-   "https://gnunet.org/configuration-dns",
-   NULL,
-   &gns_value_install_edited_handler,
-   NULL,
-   NULL,
-   NULL,
+   "GNUNET_setup_gns_auto_import_checkbutton",
+   "toggled",
+   "gns",
+   "AUTO_IMPORT_PKEY",
+   gettext_noop ("Should names be automatically learned and placed into the 'shorten' Zone?"),
+   "https://gnunet.org/configuration-gns-shortening",
+   &load_yes_no,
+   &save_yes_no, NULL,
+   NULL, NULL,
    NULL},
 
   /* END of list */
@@ -2631,4 +2012,5 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL}
 };
 
+
 /* end of gnunet-setup-options.c */
