@@ -208,8 +208,16 @@ create_plot (const char *box_name, const struct PlotInfo *info)
 static void
 shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
+  GNUNET_GTK_tray_icon_destroy ();
+  GNUNET_GTK_main_loop_quit (ml);
+  ml = NULL;
   GNUNET_STATISTICS_destroy (statistics, GNUNET_NO);
   statistics = NULL;
+  if (connection_task != GNUNET_SCHEDULER_NO_TASK)
+  {
+    GNUNET_SCHEDULER_cancel (connection_task);
+    connection_task = GNUNET_SCHEDULER_NO_TASK;
+  }
 }
 
 
@@ -219,23 +227,7 @@ shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 void
 GNUNET_STATISTICS_GTK_quit_cb (GObject * object, gpointer user_data)
 {
-  GNUNET_GTK_tray_icon_destroy ();
-  GNUNET_GTK_main_loop_quit (ml);
-  if (connection_task != GNUNET_SCHEDULER_NO_TASK)
-  {
-    GNUNET_SCHEDULER_cancel (connection_task);
-    connection_task = GNUNET_SCHEDULER_NO_TASK;
-  }
-  GNUNET_SCHEDULER_add_now (&shutdown_task, NULL);
-}
-
-/**
- * Callback invoked if the application is supposed to exit from the tray.
- */
-void
-GNUNET_GTK_quit_cb (GObject * object, gpointer user_data)
-{
-  GNUNET_STATISTICS_GTK_quit_cb (object, user_data);
+  GNUNET_SCHEDULER_shutdown ();
 }
 
 
@@ -270,7 +262,8 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   /* setup main window */
   main_window = GTK_WIDGET (get_object ("GNUNET_STATISTICS_GTK_main_window"));
   gtk_window_maximize (GTK_WINDOW (main_window));
-  GNUNET_GTK_tray_icon_create (GTK_WINDOW (main_window),
+  GNUNET_GTK_tray_icon_create (ml,
+			       GTK_WINDOW (main_window),
                                "gnunet-gtk" /* FIXME: different icon? */ ,
                                "gnunet-statistics-gtk");
 
@@ -284,6 +277,8 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     gtk_widget_show (main_window);
     gtk_window_present (GTK_WINDOW (main_window));
   }
+  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
+				&shutdown_task, NULL);
 }
 
 
